//////////////////////////////////////////////////////////////////////
//
//  Interface used by I2C devices
//
//////////////////////////////////////////////////////////////////////
#if !defined(I2C_INTERFACE_H)
#define I2C_INTERFACE_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Interface.h"

// interface names for the I2C bus

#define IFACE_I2CBUS "I2CBus"
#define IFACE_I2CBUSMASTER "I2CBusMaster"

// Interface exported by devices on an I2CBus
struct II2CDevice
{
	Interface iface;

	// Called to read a byte
	Byte (*Read)(II2CDevice*);

	// Called to write a byte
	void (*Write)(II2CDevice*, Byte data);

	// Called to select a device as active
	BOOL (*Select)(II2CDevice*, Byte address);

	// Called to deselect a device
	void (*Deselect)(II2CDevice*);
};

// Interface exported for use by I2CBus device implementations
struct II2CBus
{
	Interface iface;

	// Used to attach/remove devices from the bus at startup
	BOOL (*AttachDevice)(II2CBus*, II2CDevice*, Byte address);
	BOOL (*DetachDevice)(II2CBus*, II2CDevice*, Byte address);
};

// Interface exported for use by I2CBus users, E.g. IOC
struct II2CBusMaster
{
	Interface iface;

	// Set the state of the I2C bus lines
	void (*SetState)(II2CBusMaster*, BOOL clock, BOOL data);

	// Get the state of the I2C data line
	BOOL (*GetState)(II2CBusMaster*);
};

#endif // I2C_INTERFACE_H

