// IocPodule.h: interface for the CIocPodule class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IOCPODULE_H__80D52EE8_D1A0_4CBC_8092_9C94A01CC825__INCLUDED_)
#define AFX_IOCPODULE_H__80D52EE8_D1A0_4CBC_8092_9C94A01CC825__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SimpleDevice.h"
#include "BusInterface.h"
#include "IocInterface.h"

// Wrapper to simplify implementation of podules. Provides all the standard
// interfaces found on a podule

// Flags used during construction of the podule. 

// Declare which regions of I/O space are handled by this podule
#define PODULE_SIMPLE 1		
#define PODULE_EASI 2
#define PODULE_MEMC 4


// Podule devices derive from this rather than from CSipleDevice
// as there are separate calls for simple space and easi space
class CIocPodule : public CComponent 
{
	CSimpleDevice* simple_handler[4];	// mapped to 4 locations
	CSimpleDevice* memc_handler;
	CSimpleDevice* easi_handler;
	Word flags;
	Word podule;

	IInterrupt pod_irq;					// shared podule interrupt			
	IInterrupt pod_firq;
	IIoController* irq;					// handle to irq interrupt controller
	IIoController* firq;				// handle to firq interrupt controller

protected:
	CIocPodule(Word num=0, Word flags=PODULE_SIMPLE);

public:
	virtual ~CIocPodule();

	// CComponent interface
	virtual BOOL Init();
	virtual BOOL Cleanup();
	virtual BOOL Connect();
	virtual BOOL Disconnect();

	// Declared interface. Addresses are always relative to the base
	// of the podules section

	// This interface is the same as IPhysicalBusDevice so implementations
	// can switch between base classes. This interface is called when
	// the low io space is accessed
	virtual void WriteWord(Word address, Word data) { }
	virtual void WriteHalf(Word address, HalfWord data) { }
	virtual void WriteByte(Word address, Byte data) {  }
	virtual Word ReadWord(Word address) { return 0xffffffff; }
	virtual HalfWord ReadHalf(Word address) { return 0xffff; }
	virtual Byte ReadByte(Word address) { return 0xff; }

	// Called when 'memc/module' space is accessed
	virtual void MEMCWriteWord(Word address, Word data) { }
	virtual void MEMCWriteHalf(Word address, HalfWord data) {  }
	virtual void MEMCWriteByte(Word address, Byte data) { }
	virtual Word MEMCReadWord(Word address) { return 0xffffffff; }
	virtual HalfWord MEMCReadHalf(Word address) { return 0xffff; }
	virtual Byte MEMCReadByte(Word address) { return 0xff; }

	// Called when easi space is accessed
	virtual void EASIWriteWord(Word address, Word data) { }
	virtual void EASIWriteHalf(Word address, HalfWord data) {  }
	virtual void EASIWriteByte(Word address, Byte data) { }
	virtual Word EASIReadWord(Word address) { return 0xffffffff; }
	virtual HalfWord EASIReadHalf(Word address) { return 0xffff; }
	virtual Byte EASIReadByte(Word address) { return 0xff; }

	// Utility functions
	void SetIrq();
	void ClearIrq();
	void SetFirq();
	void ClearFirq();
};

#endif // !defined(AFX_IOCPODULE_H__80D52EE8_D1A0_4CBC_8092_9C94A01CC825__INCLUDED_)
