// PluginBusDevice.h: interface for the CPluginBusDevice class.
//
// Proxy class which passes calls on via a PlgComponent interface
// to the equivalent CPluginBusDevice of the given class.
//
//////////////////////////////////////////////////////////////////////
#if !defined(TIMERINTERFACE_H)
#define TIMERINTERFACE_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Interface.h"

// interface name for the scheduler

#define IFACE_SCHEDULER "Scheduler"

// An event structure used for timed callbacks
struct IEvent
{
	Interface iface;

	Word handle;	// internal event handle

	// called when an event times out. Delta is the overrun in ticks
	// of when the event was due. It _may_ be negative.
	// The return value from this should be +the number of ticks
	// before this event should be called again or 0 to cancel the 
	// event
	Word (*ProcessEvent)(IEvent*, int delta);
};

// Exported by the system scheduler. Enables timed callbacks to be
// registered.
struct IScheduler
{
	Interface iface;

	// Schedule an event object to be called in so many ticks time.
	Word (*ScheduleEvent)(IScheduler*, IEvent*, Word ticks);

	// Deschedule a previously scheduled event
	BOOL (*DescheduleEvent)(IScheduler*, IEvent*);

	// Reschedule a previously scheduled event. This should only be
	// called synchronously
	BOOL (*RescheduleEvent)(IScheduler*, IEvent*, Word ticks);

	// Dont call this
	void (*ResetEvent)(IScheduler*, IEvent*);

	// Find out how long before the event will be triggered
	Word (*TicksToGo)(IScheduler*, IEvent*);

	// How many ticks there are per second. This will not change during
	// any given invocation of the emulator
	Word (*TickRate)(IScheduler*);			 // in hz. Typically 2Mhz

	// The general granularity of the scheduler. It varies depending on
	// host machine and speed of emulation. The result is in ticks. There
	// is no point scheduling events to happen more often than this
	// granularity.
	Word (*TickResolution)(IScheduler*);
};

#endif // TIMERINTERFACE_H